#ifndef S60_H
#define S60_H

#define VERSION "3.3.2"
#define DATAPATH "./"
#define STEAD_PATH DATAPATH"stead/"
#define THEMES_PATH DATAPATH"themes/"
#define GAMES_PATH DATAPATH"games/"
#define ICON_PATH DATAPATH"icon/"
#define LANG_PATH DATAPATH"lang/"

#endif
